package ru.crabs.sjson.language

import com.intellij.extapi.psi.PsiFileBase
import com.intellij.lang.ASTNode
import com.intellij.lang.ParserDefinition
import com.intellij.lang.PsiParser
import com.intellij.lexer.FlexAdapter
import com.intellij.lexer.Lexer
import com.intellij.openapi.fileTypes.FileType
import com.intellij.openapi.project.Project
import com.intellij.psi.FileViewProvider
import com.intellij.psi.PsiElement
import com.intellij.psi.PsiFile
import com.intellij.psi.tree.IFileElementType
import com.intellij.psi.tree.TokenSet
import ru.crabs.sdk.language.parser.SJsonParser
import ru.crabs.sdk.language.psi.SJsonTypes
import ru.crabs.sjson.SJsonFileType
import ru.crabs.sjson.SJsonLanguage


class SJsonLexerAdapter : FlexAdapter(SJsonLexer(null))

class SJsonFile(viewProvider: FileViewProvider) :

    PsiFileBase(viewProvider, SJsonLanguage.INSTANCE) {

    override fun getFileType(): FileType {
        return SJsonFileType.INSTANCE
    }

    override fun toString(): String {
        return "SJson File"
    }
}

interface SJsonTokenSets {
    companion object {
        val VARS_WORD: TokenSet = TokenSet.create(SJsonTypes.VARS_WORD)

        var COMMENTS: TokenSet = TokenSet.create(SJsonTypes.COMMENT)
    }
}

internal class SJsonParserDefinition : ParserDefinition {

    override fun createLexer(project: Project?): Lexer {
        return SJsonLexerAdapter()
    }

    override fun getCommentTokens(): TokenSet {
        return SJsonTokenSets.COMMENTS
    }

    override fun getStringLiteralElements(): TokenSet {
        return TokenSet.EMPTY
    }

    override fun createParser(project: Project?): PsiParser {
        return SJsonParser()
    }

    override fun getFileNodeType(): IFileElementType {
        return FILE
    }

    override fun createFile(viewProvider: FileViewProvider): PsiFile {
        return SJsonFile(viewProvider)
    }

    override fun createElement(node: ASTNode?): PsiElement {
        return SJsonTypes.Factory.createElement(node)
    }

    companion object {
        val FILE: IFileElementType = IFileElementType(SJsonLanguage.INSTANCE)
    }
}
